@@ -20,6 +20,7 @@ package Fedora::Bugzilla::Bug;
 use Moose;
 
 use MooseX::AttributeHelpers;
+use MooseX::CascadeClearing;
 use Moose::Util::TypeConstraints;
 use MooseX::Types::DateTimeX qw{ DateTime };
 use MooseX::Types::Path::Class;
@@ -43,17 +44,38 @@ use namespace::clean -except => 'meta';
 
 use overload '""' => sub { shift->id }, fallback => 1;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 ########################################################################
 # parent Fedora::Bugzilla 
 
-has bz => ( is => 'ro', isa => 'Fedora::Bugzilla', required => 1);
+has bz => (is => 'ro', isa => 'Fedora::Bugzilla', required => 1);
 
 ########################################################################
-# our types / coercions
+# Handle the alias on construction correctly 
 
-# now in Fedora::Bugzilla::Types!
+around BUILDARGS => sub {
+    my $orig  = shift @_;
+    my $class = shift @_; 
+    
+    ### in BUILDARGS...
+    ##### @_
+
+    if (@_ > 1 || ref $_[0] eq 'HASH') {
+
+        my $args = @_ > 1 ? { @_ } : $_[0];
+
+        if (exists $args->{alias}) {
+            
+            $args->{_aliases} = { $args->{alias} => 1 };
+            delete $args->{alias};
+            ##### $args
+            return $class->$orig($args);
+        }
+    } 
+
+    return $class->$orig(@_); 
+}; 
 
 ########################################################################
 # data: the meat of it 
@@ -62,22 +84,17 @@ has bz => ( is => 'ro', isa => 'Fedora::Bugzilla', required => 1);
 # that if any updates are made, this is NOT the place to do it; update() pulls
 # the new values from the attributes themselves, NOT this hash.
 
-has data => ( 
-    traits => [ 'CascadeClearMaster' ],
-
-    is         => 'ro', 
-    isa        => 'HashRef', 
-    lazy_build => 1,
-);
+has data =>
+    (is => 'ro', isa => 'HashRef', lazy_build => 1, is_clear_master => 1);
 
 sub _build_data {
     my $self = shift @_;
 
     # prefer id over alias
     my $emsg = 'Neither bug id nor alias has been provided';
-    my $bug_id = $self->has_id    ? $self->id 
-               : $self->has_alias ? $self->alias
-               :                    confess $emsg
+    my $bug_id = $self->has_id       ? $self->id 
+               : $self->_has_aliases ? $self->alias
+               :                       confess $emsg
                ;
 
     my $ret_hash = $self->bz->rpc->simple_request(
@@ -93,8 +110,7 @@ sub refresh { shift->clear_data }
 
 # set true when we need to do an update
 has dirty => (
-    traits   => [ 'CascadeClear' ],
-    clear_on => 'data',
+    clear_master => 'data',
     clearer  => 'clear_dirty',
     is       => 'rw', 
     isa      => 'Bool', 
@@ -106,6 +122,12 @@ has dirty => (
 sub _dirty_trigger  {
     my ($self, $new_value, $meta) = @_;
 
+    ### $new_value
+    ### $meta
+
+    # FIXME not exactly sure...
+    return unless $meta;
+
     $self->dirty(1);
     $self->_to_update($meta->name);
 }
@@ -119,7 +141,8 @@ sub update {
 
     # force stringification
     my %updates = 
-        map { my $x = $self->$_ || q{}; $_ => "$x" } $self->_update_these;
+        #map { my $x = $self->$_ || q{}; $_ => "$x" } $self->_update_these;
+        map { my $x = $self->$_ || q{}; $_ => blessed $x ? "$x" : $x } $self->_update_these;
     
     ### %updates
 
@@ -144,16 +167,14 @@ sub update {
 
 # default attribute attributes :-)
 my @defaults = (
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'data',
+    clear_master   => 'data',
     is         => 'ro', 
     isa        => 'Str', 
     lazy_build => 1,
 );
 
 my @rw_defaults = ( 
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'data',
+    clear_master   => 'data',
 
     is         => 'rw', 
     isa        => 'Str', 
@@ -162,8 +183,7 @@ my @rw_defaults = (
 );
 
 my @dt_defaults = (
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'data',
+    clear_master   => 'data',
 
     is         => 'ro', 
     # FIXME hm.
@@ -177,9 +197,15 @@ my @dt_defaults = (
 # actual bug attributes 
 
 has id => ( 
-    traits    => [ 'MooseX::MultiInitArg::Trait' ],
+    # seems to fail at mixing in existing metaclass traits??
+    #traits    => [ 'MooseX::MultiInitArg::Trait' ],
+    traits    => [ 
+        'MooseX::MultiInitArg::Trait',
+        'MooseX::CascadeClearing::Role::Meta::Attribute',
+        #'MooseX::AttributeHelpers::Trait::Collection::List',
+    ],
     init_args => [ 'bug_id' ],
-    clear_on  => 'data',
+    clear_master  => 'data',
     is        => 'ro', 
     isa       => 'Int', 
     lazy      => 1, 
@@ -192,21 +218,50 @@ sub _build_id {
     my $self = shift @_;
 
     confess 'Must set either id or alias!'
-        if not $self->has_alias;
+        if not $self->_has_aliases;
 
     return $self->data->{id};
 }
 
-has alias => (
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'data',
+has _aliases => (
+    traits     => [ 
+        'MooseX::AttributeHelpers::Trait::Collection::Hash',
+    ],
     is         => 'rw',
-    isa        => 'Maybe[Str20]',
+    #isa        => 'ArrayRef[Str20]',
+    isa        => 'HashRef',
     lazy_build => 1,
     trigger    => \&_dirty_trigger,
+
+    provides => {
+        count  => 'num_aliases',
+        keys   => 'aliases',
+        #add    => 'add_alias',
+        set    => 'add_alias',
+        delete => 'delete_alias',
+        exists => 'has_alias',
+        empty  => 'has_aliases',
+    },
 );
 
-sub _build_alias { shift->data->{alias} }
+# we should warn, but I haven't actually seen any multi-alias bugs yet
+#sub alias { ($_[0]->aliases)[0] if $_[0]->has_aliases }
+sub alias { 
+    my ($self, $value) = @_;
+
+    $self->_aliases({ $value => 1 }) if defined $value;
+    return ($_[0]->aliases)[0] if $_[0]->has_aliases; 
+}
+
+#sub _build__aliases { { map { $_ => 1 } @{shift->data->{alias}} } }
+sub _build__aliases { 
+
+    my $self = shift @_;
+    my $data = $self->data->{alias};
+
+    ### $data
+    return { map { $_ => 1 } @{$self->data->{alias}} };
+}
 
 #sub __builder { shift->data->{shift} }
 #sub __internals_builder { shift->data->{internals}->{shift} }
@@ -217,9 +272,16 @@ sub _build_alias { shift->data->{alias} }
 has summary => (
     @rw_defaults, 
 
-    traits    => [ 'CascadeClear', 'MooseX::MultiInitArg::Trait' ],
+    # seems to fail at mixing in existing metaclass traits??
+    #traits    => [ 'MooseX::MultiInitArg::Trait' ],
+    traits    => [ 
+        'MooseX::MultiInitArg::Trait',
+        'MooseX::CascadeClearing::Role::Meta::Attribute',
+        #'MooseX::AttributeHelpers::Trait::Collection::List',
+    ],
+    init_args => [ 'bug_id' ],
     init_args => [ 'short_desc' ],
-    clear_on  => 'data',
+    clear_master  => 'data',
 
     is         => 'rw', 
     isa        => 'Str', 
@@ -272,12 +334,11 @@ sub _build_full_status {
 }
     
 has _update_these => (
-    metaclass => 'Collection::Array',
-    traits    => [ 'CascadeClear' ],
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::Array' ],
 
     is         => 'rw',
     isa        => 'ArrayRef[Str]',
-    clear_on   => 'data',
+    clear_master   => 'data',
     lazy       => 1,
     auto_deref => 1,
 
@@ -299,11 +360,11 @@ has _update_these => (
 # particular bits.
 
 has xml => (
-    traits     => [ 'CascadeClear', 'CascadeClearMaster' ],
-    clear_on   => 'data',
-    is         => 'ro', 
-    isa        => 'Str', 
-    lazy_build => 1,
+    clear_master    => 'data',
+    is_clear_master => 1,
+    is              => 'ro', 
+    isa             => 'Str', 
+    lazy_build      => 1,
 );
 
 sub _build_xml {
@@ -326,8 +387,7 @@ sub _build_xml {
 
 # parse and build the twig on demand
 has twig => (
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'xml',
+    clear_master   => 'xml',
     is         => 'ro', 
     isa        => 'XML::Twig', 
     lazy_build => 1,
@@ -349,13 +409,9 @@ sub _from_atts {
 # <flag name="fedora-review" status="+" setter="kwizart@gmail.com" />
 
 has _flags => (
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::ImmutableHash' ],
 
-    # note this is not the place to set flags :)
-    metaclass => 'Collection::ImmutableHash',
-
-    # I think this should "just work"
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'xml',
+    clear_master => 'xml',
 
     is  => 'ro',
     isa => 'HashRef[Fedora::Bugzilla::Bug::Flag]',
@@ -406,11 +462,10 @@ sub _build__flags {
 }
 
 has _uris => (
-    metaclass => 'Collection::List',
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::List' ],
     
     # I think this should "just work"
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'xml',
+    clear_master   => 'xml',
 
     is  => 'ro',
     isa => 'ArrayRef[URI]',
@@ -447,10 +502,9 @@ sub _build__uris {
 }
 
 has _comments => (
-    metaclass => 'Collection::List',
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::List' ],
     
-    traits   => [ 'CascadeClear' ],
-    clear_on => 'xml',
+    clear_master => 'xml',
 
     is         => 'ro',
     isa        => 'ArrayRef[Fedora::Bugzilla::Bug::Comment]',
@@ -492,10 +546,9 @@ sub _build__comments {
 }
 
 has _attachments => (
-    metaclass => 'Collection::List',
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::List' ],
     
-    traits   => [ 'CascadeClear' ],
-    clear_on => 'xml',
+    clear_master => 'xml',
 
     is         => 'ro',
     isa        => 'ArrayRef[Fedora::Bugzilla::Bug::Attachment]',
@@ -538,9 +591,8 @@ sub _build__attachments {
 }
 
 has _dependson => (
-    metaclass  => 'Collection::Bag',
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'xml',
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::Bag' ],
+    clear_master   => 'xml',
     # FIXME trigger on set needed
     is         => 'ro', 
     isa        => 'Bag', 
@@ -561,9 +613,8 @@ has _dependson => (
 );
 
 has _blocked => (
-    metaclass  => 'Collection::Bag',
-    traits     => [ 'CascadeClear' ],
-    clear_on   => 'xml',
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::Bag' ],
+    clear_master   => 'xml',
     # FIXME trigger on set needed
     is         => 'ro', 
     isa        => 'Bag', 
@@ -589,10 +640,9 @@ sub _build__blocked
     { return { map { $_ => 1 } @{ shift->_from_atts('blocked')   } } }
 
 has cc_list => (
-    metaclass => 'Collection::List',
-    traits    => [ 'CascadeClear' ],
+    traits => [ 'MooseX::AttributeHelpers::Trait::Collection::List' ],
 
-    clear_on => 'xml',
+    clear_master => 'xml',
 
     is  => 'ro',
     #isa => 'ArrayRef[EmailAddress]',
